@@ -1,7 +1,9 @@
 import colorsys
 from enum import Enum
 import logging
+import random
 import time
+from typing import Dict, Iterable, List, Optional, Set, Tuple, Type, Union
 
 from .const import (  # imported for back compat, remove once Home Assistant no longer uses
     ADDRESSABLE_STATE_CHANGE_LATENCY,
@@ -15,6 +17,7 @@ from .const import (  # imported for back compat, remove once Home Assistant no
     COLOR_MODES_RGB_CCT,
     COLOR_MODES_RGB_W,
     DEFAULT_MODE,
+    EFFECT_RANDOM,
     MAX_TEMP,
     MIN_TEMP,
     MODE_COLOR,
@@ -36,17 +39,7 @@ from .const import (  # imported for back compat, remove once Home Assistant no
     WRITE_ALL_WHITES,
     LevelWriteMode,
 )
-from .models_db import (
-    ADDRESSABLE_MODELS,
-    BASE_MODE_MAP,
-    CHANNEL_REMAP,
-    MODEL_DESCRIPTIONS,
-    MODEL_MAP,
-    ORIGINAL_ADDRESSABLE_MODELS,
-    RGBW_PROTOCOL_MODELS,
-    UNKNOWN_MODEL,
-    USE_9BYTE_PROTOCOL_MODELS,
-)
+from .models_db import BASE_MODE_MAP, LEDENETModel, get_model, is_known_model
 from .pattern import (
     ADDRESSABLE_EFFECT_ID_NAME,
     ADDRESSABLE_EFFECT_NAME_ID,
@@ -61,22 +54,77 @@ from .pattern import (
 )
 from .protocol import (
     PROTOCOL_LEDENET_8BYTE,
+    PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS,
     PROTOCOL_LEDENET_9BYTE,
-    PROTOCOL_LEDENET_ADDRESSABLE,
+    PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS,
+    PROTOCOL_LEDENET_ADDRESSABLE_A1,
+    PROTOCOL_LEDENET_ADDRESSABLE_A2,
+    PROTOCOL_LEDENET_ADDRESSABLE_A3,
+    PROTOCOL_LEDENET_CCT,
     PROTOCOL_LEDENET_ORIGINAL,
-    PROTOCOL_LEDENET_ORIGINAL_ADDRESSABLE,
+    LEDENETOriginalRawState,
+    LEDENETRawState,
     ProtocolLEDENET8Byte,
+    ProtocolLEDENET8ByteDimmableEffects,
     ProtocolLEDENET9Byte,
-    ProtocolLEDENETAddressable,
+    ProtocolLEDENET9ByteDimmableEffects,
+    ProtocolLEDENETAddressableA1,
+    ProtocolLEDENETAddressableA2,
+    ProtocolLEDENETAddressableA3,
+    ProtocolLEDENETCCT,
     ProtocolLEDENETOriginal,
-    ProtocolLEDENETOriginalAddressable,
 )
 from .timer import BuiltInTimer
-from .utils import utils, white_levels_to_color_temp
+from .utils import scaled_color_temp_to_white_levels, utils, white_levels_to_color_temp
 
 _LOGGER = logging.getLogger(__name__)
 
 
+PROTOCOL_PROBES: Tuple[Type[ProtocolLEDENET8Byte], Type[ProtocolLEDENETOriginal]] = (
+    ProtocolLEDENET8Byte,
+    ProtocolLEDENETOriginal,
+)
+
+PROTOCOL_TYPES = Union[
+    ProtocolLEDENET8Byte,
+    ProtocolLEDENET8ByteDimmableEffects,
+    ProtocolLEDENET9Byte,
+    ProtocolLEDENET9ByteDimmableEffects,
+    ProtocolLEDENETAddressableA1,
+    ProtocolLEDENETAddressableA2,
+    ProtocolLEDENETAddressableA3,
+    ProtocolLEDENETOriginal,
+    ProtocolLEDENETCCT,
+]
+
+ADDRESSABLE_PROTOCOLS = {
+    PROTOCOL_LEDENET_ADDRESSABLE_A1,
+    PROTOCOL_LEDENET_ADDRESSABLE_A2,
+    PROTOCOL_LEDENET_ADDRESSABLE_A3,
+}
+
+OLD_EFFECTS_PROTOCOLS = {PROTOCOL_LEDENET_ADDRESSABLE_A1}
+NEW_EFFECTS_PROTOCOLS = {
+    PROTOCOL_LEDENET_ADDRESSABLE_A2,
+    PROTOCOL_LEDENET_ADDRESSABLE_A3,
+}
+SPEED_ADJUST_WILL_TURN_ON = {
+    PROTOCOL_LEDENET_ADDRESSABLE_A1,
+    PROTOCOL_LEDENET_ADDRESSABLE_A2,
+}
+PROTOCOL_NAME_TO_CLS = {
+    PROTOCOL_LEDENET_ORIGINAL: ProtocolLEDENETOriginal,
+    PROTOCOL_LEDENET_8BYTE: ProtocolLEDENET8Byte,
+    PROTOCOL_LEDENET_8BYTE_DIMMABLE_EFFECTS: ProtocolLEDENET8ByteDimmableEffects,
+    PROTOCOL_LEDENET_9BYTE: ProtocolLEDENET9Byte,
+    PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS: ProtocolLEDENET9ByteDimmableEffects,
+    PROTOCOL_LEDENET_ADDRESSABLE_A3: ProtocolLEDENETAddressableA3,
+    PROTOCOL_LEDENET_ADDRESSABLE_A2: ProtocolLEDENETAddressableA2,
+    PROTOCOL_LEDENET_ADDRESSABLE_A1: ProtocolLEDENETAddressableA1,
+    PROTOCOL_LEDENET_CCT: ProtocolLEDENETCCT,
+}
+
+
 class DeviceType(Enum):
     Bulb = 0
     Switch = 1
@@ -85,117 +133,132 @@ class DeviceType(Enum):
 class LEDENETDevice:
     """An LEDENET Device."""
 
-    def __init__(self, ipaddr, port=5577, timeout=5):
+    def __init__(self, ipaddr: str, port: int = 5577, timeout: float = 5) -> None:
         """Init the LEDENEt Device."""
-        self.ipaddr = ipaddr
-        self.port = port
-        self.timeout = timeout
-        self.raw_state = None
-        self.available = None
-        self._protocol = None
-        self._mode = None
-        self._transition_complete_time = 0
+        self.ipaddr: str = ipaddr
+        self.port: int = port
+        self.timeout: float = timeout
+        self.raw_state: Optional[Union[LEDENETOriginalRawState, LEDENETRawState]] = None
+        self.available: Optional[bool] = None
+        self._model_num: Optional[int] = None
+        self._model_data: Optional[LEDENETModel] = None
+        self._protocol: Optional[PROTOCOL_TYPES] = None
+        self._mode: Optional[str] = None
+        self._transition_complete_time: float = 0
+        self._last_effect_brightness: int = 100
 
     @property
-    def model_num(self):
+    def model_num(self) -> int:
         """Return the model number."""
-        return self.raw_state.model_num if self.raw_state else None
+        assert self._model_num is not None
+        return self._model_num
 
     @property
-    def model(self):
+    def model_data(self) -> LEDENETModel:
+        """Return the model data."""
+        assert self._model_data is not None
+        return self._model_data
+
+    @property
+    def speed_adjust_off(self) -> int:
+        """Return true if the speed of an effect can be adjusted while off."""
+        return self.protocol not in SPEED_ADJUST_WILL_TURN_ON
+
+    @property
+    def _whites_are_temp_brightness(self) -> bool:
+        """Return true if warm_white and cool_white are scaled temp values and not raw 0-255."""
+        return self.protocol == PROTOCOL_LEDENET_CCT
+
+    @property
+    def model(self) -> str:
         """Return the human readable model description."""
-        model_num = self.model_num
-        description = MODEL_DESCRIPTIONS.get(model_num) or UNKNOWN_MODEL
-        return f"{description} (0x{model_num:02X})"
+        return f"{self.model_data.description} (0x{self.model_num:02X})"
 
     @property
-    def version_num(self):
+    def version_num(self) -> int:
         """Return the version number."""
+        assert self.raw_state is not None
         raw_state = self.raw_state
-        return raw_state.version_number if hasattr(raw_state, "version_number") else 1
+        if hasattr(raw_state, "version_number"):
+            assert isinstance(raw_state, LEDENETRawState)
+            return raw_state.version_number
+        return 1
 
     @property
-    def preset_pattern_num(self):
+    def preset_pattern_num(self) -> int:
         """Return the preset pattern number."""
+        assert self.raw_state is not None
         return self.raw_state.preset_pattern
 
     @property
-    def rgbwprotocol(self):
+    def rgbwprotocol(self) -> bool:
         """Devices that don't require a separate rgb/w bit."""
-        return self.model_num in RGBW_PROTOCOL_MODELS
+        return self.model_data.always_writes_white_and_colors
 
     @property
-    def addressable(self):
-        """Devices that have addressable leds."""
-        return self._is_addressable(self.model_num)
-
-    def _is_addressable(self, model_num):
-        """Devices that have addressable leds."""
-        return model_num in ADDRESSABLE_MODELS
+    def microphone(self) -> bool:
+        """Devices that have a microphone built in."""
+        return self.model_data.microphone
 
     @property
-    def original_addressable(self):
-        """Devices that have addressable leds using the original addressable protocol."""
-        return self._is_original_addressable(self.model_num)
-
-    def _is_original_addressable(self, model_num):
-        """Devices that have addressable leds using the original addressable protocol."""
-        return model_num in ORIGINAL_ADDRESSABLE_MODELS
-
-    @property
-    def rgbwcapable(self):
+    def rgbwcapable(self) -> bool:
         """Devices that actually support rgbw."""
         color_modes = self.color_modes
         return COLOR_MODE_RGBW in color_modes or COLOR_MODE_RGBWW in color_modes
 
     @property
-    def device_type(self):
+    def device_type(self) -> DeviceType:
         """Return the device type."""
         is_switch = self.model_num in MODEL_NUMS_SWITCHS
         return DeviceType.Switch if is_switch else DeviceType.Bulb
 
     @property
-    def color_temp(self):
+    def color_temp(self) -> int:
         """Return the current color temp in kelvin."""
         return (self.getWhiteTemperature())[0]
 
     @property
-    def min_temp(self):
+    def min_temp(self) -> int:
         """Returns the minimum color temp in kelvin."""
         return MIN_TEMP
 
     @property
-    def max_temp(self):
+    def max_temp(self) -> int:
         """Returns the maximum color temp in kelvin."""
         return MAX_TEMP
 
     @property
-    def _rgbwwprotocol(self):
+    def _rgbwwprotocol(self) -> bool:
         """Device that uses the 9-byte protocol."""
-        return self._uses_9byte_protocol(self.model_num)
-
-    def _uses_9byte_protocol(self, model_num):
-        """Devices that use a 9-byte protocol."""
-        return model_num in USE_9BYTE_PROTOCOL_MODELS
+        return self.protocol in (
+            PROTOCOL_LEDENET_9BYTE,
+            PROTOCOL_LEDENET_9BYTE_DIMMABLE_EFFECTS,
+        )
 
     @property
-    def white_active(self):
+    def white_active(self) -> bool:
         """Any white channel is active."""
-        return bool(self.raw_state.warm_white or self.raw_state.cool_white)
+        assert self.raw_state is not None
+        raw_state = self.raw_state
+        if hasattr(raw_state, "cool_white"):
+            assert isinstance(raw_state, LEDENETRawState)
+            return bool(raw_state.warm_white or raw_state.cool_white)
+        return bool(raw_state.warm_white)
 
     @property
-    def color_active(self):
+    def color_active(self) -> bool:
         """Any color channel is active."""
+        assert self.raw_state is not None
         raw_state = self.raw_state
         return bool(raw_state.red or raw_state.green or raw_state.blue)
 
     @property
-    def multi_color_mode(self):
+    def multi_color_mode(self) -> bool:
         """The device supports multiple color modes."""
         return len(self.color_modes) > 1
 
     @property
-    def color_modes(self):
+    def color_modes(self) -> Set[str]:
         """The available color modes."""
         color_modes = self._internal_color_modes
         # We support CCT mode if the device supports RGBWW
@@ -207,18 +270,19 @@ class LEDENETDevice:
         return color_modes
 
     @property
-    def _internal_color_modes(self):
+    def _internal_color_modes(self) -> Set[str]:
         """The internal available color modes."""
-        model_db_entry = MODEL_MAP.get(self.model_num)
-        if not model_db_entry:
+        assert self.raw_state is not None
+        if not is_known_model(self.model_num):
             # Default mode is RGB
             return BASE_MODE_MAP.get(self.raw_state.mode & 0x0F, {DEFAULT_MODE})
-        return model_db_entry.mode_to_color_mode.get(
-            self.raw_state.mode, model_db_entry.color_modes
+        model_data = self.model_data
+        return model_data.mode_to_color_mode.get(
+            self.raw_state.mode, model_data.color_modes
         )
 
     @property
-    def color_mode(self):
+    def color_mode(self) -> Optional[str]:
         """The current color mode."""
         color_modes = self._internal_color_modes
         if COLOR_MODE_RGBWW in color_modes:
@@ -235,46 +299,61 @@ class LEDENETDevice:
         return None  # Usually a switch or non-light device
 
     @property
-    def protocol(self):
+    def protocol(self) -> Optional[str]:
         """Returns the name of the protocol in use."""
-        if not self._protocol:
+        if self._protocol is None:
             return None
         return self._protocol.name
 
     @property
-    def is_on(self):
+    def dimmable_effects(self) -> bool:
+        """Return true of the device supports dimmable effects."""
+        assert self._protocol is not None
+        return self._protocol.dimmable_effects
+
+    @property
+    def is_on(self) -> bool:
+        assert self.raw_state is not None
+        assert self._protocol is not None
         return self.raw_state.power_state == self._protocol.on_byte
 
     @property
-    def mode(self):
+    def mode(self) -> Optional[str]:
         return self._mode
 
     @property
-    def warm_white(self):
+    def warm_white(self) -> int:
+        assert self.raw_state is not None
         return self.raw_state.warm_white if self._rgbwwprotocol else 0
 
     @property
-    def effect_list(self):
+    def effect_list(self) -> List[str]:
         """Return the list of available effects."""
-        if self.original_addressable:
-            return ORIGINAL_ADDRESSABLE_EFFECT_ID_NAME.values()
-        if self.addressable:
-            return ADDRESSABLE_EFFECT_ID_NAME.values()
-        return EFFECT_LIST
+        effects: Iterable[str] = []
+        protocol = self.protocol
+        if protocol in OLD_EFFECTS_PROTOCOLS:
+            effects = ORIGINAL_ADDRESSABLE_EFFECT_ID_NAME.values()
+        elif protocol in NEW_EFFECTS_PROTOCOLS:
+            effects = ADDRESSABLE_EFFECT_ID_NAME.values()
+        elif COLOR_MODES_RGB.intersection(self.color_modes):
+            effects = EFFECT_LIST
+        return [*effects, EFFECT_RANDOM]
 
     @property
-    def effect(self):
+    def effect(self) -> Optional[str]:
         """Return the current effect."""
+        assert self.raw_state is not None
         pattern_code = self.preset_pattern_num
         if pattern_code == PRESET_MUSIC_MODE:
             return MODE_MUSIC
         if pattern_code == EFFECT_CUSTOM_CODE:
             return EFFECT_CUSTOM
         mode = self.raw_state.mode
-        if self.original_addressable:
+        protocol = self.protocol
+        if protocol in OLD_EFFECTS_PROTOCOLS:
             effect_id = (pattern_code << 8) + mode - 99
             return ORIGINAL_ADDRESSABLE_EFFECT_ID_NAME.get(effect_id)
-        if self.addressable:
+        if protocol in NEW_EFFECTS_PROTOCOLS:
             if pattern_code == 0x25:
                 return ADDRESSABLE_EFFECT_ID_NAME.get(mode)
             if pattern_code == 0x24:
@@ -282,16 +361,20 @@ class LEDENETDevice:
         return EFFECT_ID_NAME.get(pattern_code)
 
     @property
-    def cool_white(self):
-        return self.raw_state.cool_white if self._rgbwwprotocol else 0
+    def cool_white(self) -> int:
+        assert self.raw_state is not None
+        if self._rgbwwprotocol:
+            assert isinstance(self.raw_state, LEDENETRawState)
+            return self.raw_state.cool_white
+        return 0
 
     # Old name is deprecated
     @property
-    def cold_white(self):
+    def cold_white(self) -> int:
         return self.cool_white
 
     @property
-    def brightness(self):
+    def brightness(self) -> int:
         """Return current brightness 0-255.
 
         For warm white return current led level. For RGB
@@ -301,7 +384,18 @@ class LEDENETDevice:
         """
         color_mode = self.color_mode
         raw_state = self.raw_state
+        assert raw_state is not None
 
+        if self._mode == MODE_PRESET:
+            if self.dimmable_effects:
+                if (
+                    self.protocol in NEW_EFFECTS_PROTOCOLS
+                    and time.monotonic() > self._transition_complete_time
+                ):
+                    # the red byte holds the brightness during an effect
+                    return min(255, round(raw_state.red * 255 / 100))
+                return round(self._last_effect_brightness * 255 / 100)
+            return 255
         if color_mode == COLOR_MODE_DIM:
             return int(raw_state.warm_white)
         elif color_mode == COLOR_MODE_CCT:
@@ -314,12 +408,14 @@ class LEDENETDevice:
         if color_mode == COLOR_MODE_RGBW:
             return round((v_255 + raw_state.warm_white) / 2)
         if color_mode == COLOR_MODE_RGBWW:
+            assert isinstance(raw_state, LEDENETRawState)
             return round((v_255 + raw_state.warm_white + raw_state.cool_white) / 3)
 
         # Default color mode (RGB)
         return int(v_255)
 
-    def _determineMode(self):
+    def _determineMode(self) -> Optional[str]:
+        assert self.raw_state is not None
         pattern_code = self.raw_state.preset_pattern
         if self.device_type == DeviceType.Switch:
             return MODE_SWITCH
@@ -335,17 +431,19 @@ class LEDENETDevice:
             return MODE_PRESET
         elif BuiltInTimer.valid(pattern_code):
             return BuiltInTimer.valtostr(pattern_code)
-        elif self.addressable or self.original_addressable:
+        elif self.protocol in ADDRESSABLE_PROTOCOLS:
             return MODE_PRESET
         return None
 
-    def set_unavailable(self):
+    def set_unavailable(self) -> None:
         self.available = False
 
-    def set_available(self):
+    def set_available(self) -> None:
         self.available = True
 
-    def process_state_response(self, rx):
+    def process_state_response(self, rx: bytes) -> bool:
+        assert self._protocol is not None
+
         if not self._protocol.is_valid_state_response(rx):
             _LOGGER.warning(
                 "%s: Recieved invalid response: %s",
@@ -354,7 +452,9 @@ class LEDENETDevice:
             )
             return False
 
-        raw_state = self._protocol.named_raw_state(rx)
+        raw_state: Union[
+            LEDENETOriginalRawState, LEDENETRawState
+        ] = self._protocol.named_raw_state(rx)
         _LOGGER.debug("%s: State: %s", self.ipaddr, raw_state)
 
         if raw_state != self.raw_state:
@@ -379,6 +479,8 @@ class LEDENETDevice:
         else:
             self._set_raw_state(raw_state)
 
+        _LOGGER.debug("%s: Mapped State: %s", self.ipaddr, self.raw_state)
+
         mode = self._determineMode()
 
         if mode is None:
@@ -392,8 +494,9 @@ class LEDENETDevice:
         self._mode = mode
         return True
 
-    def process_power_state_response(self, msg):
+    def process_power_state_response(self, msg: bytes) -> bool:
         """Process a power state change message."""
+        assert self._protocol is not None
         if not self._protocol.is_valid_power_state_response(msg):
             _LOGGER.warning(
                 "%s: Recieved invalid power state response: %s",
@@ -405,35 +508,64 @@ class LEDENETDevice:
         self._set_power_state(msg[2])
         return True
 
-    def _set_raw_state(self, raw_state, updated=None):
-        """Set the raw state remapping channels as needed."""
-        channel_map = CHANNEL_REMAP.get(raw_state.model_num)
-        if not channel_map:  # Remap channels
-            self.raw_state = raw_state
-            return
+    def _set_raw_state(
+        self,
+        raw_state: Union[LEDENETOriginalRawState, LEDENETRawState],
+        updated: Optional[Set[str]] = None,
+    ) -> None:
+        """Set the raw state remapping channels as needed.
+
+        The goal is to normalize the data so the raw state
+        is always in the same format reguardless of the protocol
+
+        Some devices need to have channels remapped
+
+        Other devices uses color_temp/brightness format
+        which needs to be converted back to 0-255 values for
+        warm_white and cool_white
+        """
+        channel_map = self.model_data.channel_map
         # Only remap updated states as we do not want to switch any
         # state that have not changed since they will already be in
         # the correct slot
         #
         # If updated is None than all raw_state values have been sent
         #
-        if updated is None:
-            updated = set(channel_map.keys())
-        self.raw_state = raw_state._replace(
-            **{
-                name: getattr(raw_state, source)
-                if source in updated
-                else getattr(raw_state, name)
-                for name, source in channel_map.items()
-            }
-        )
-        _LOGGER.debug(
-            "%s: remapped raw state: %s",
-            self.ipaddr,
-            utils.raw_state_to_dec(self.raw_state),
-        )
+        if self._whites_are_temp_brightness:
+            assert isinstance(raw_state, LEDENETRawState)
+            # Only convert on a full update since we still use 0-255 internally
+            if updated is not None:
+                self.raw_state = raw_state
+                return
+            # warm_white is the color temp from 1-100
+            temp = raw_state.warm_white
+            # cold_white is the brightness from 1-100
+            brightness = raw_state.cool_white
+            warm_white, cool_white = scaled_color_temp_to_white_levels(temp, brightness)
+            self.raw_state = raw_state._replace(
+                warm_white=warm_white, cool_white=cool_white
+            )
+            return
+
+        if channel_map:
+            if updated is None:
+                updated = set(channel_map.keys())
+            self.raw_state = raw_state._replace(
+                **{
+                    name: getattr(raw_state, source)
+                    if source in updated
+                    else getattr(raw_state, name)
+                    for name, source in channel_map.items()
+                }
+            )
+            return
+
+        self.raw_state = raw_state
+
+    def __str__(self) -> str:  # noqa: C901
+        assert self.raw_state is not None
+        assert self._protocol is not None
 
-    def __str__(self):  # noqa: C901
         rx = self.raw_state
         if not rx:
             return "No state data"
@@ -455,13 +587,13 @@ class LEDENETDevice:
                 if self.rgbwcapable:
                     mode_str += f" White: {rx.warm_white}"
                 else:
-                    mode_str += f" Brightness: {self.brightness}"
+                    mode_str += f" Brightness: {round(self.brightness * 100 / 255)}%"
             elif color_mode == COLOR_MODE_DIM:
                 mode_str = f"Warm White: {utils.byteToPercent(rx.warm_white)}%"
             elif color_mode == COLOR_MODE_CCT:
                 cct_value = self.getWhiteTemperature()
                 mode_str = "CCT: {}K Brightness: {}%".format(
-                    cct_value[0], cct_value[1] / 255
+                    cct_value[0], round(cct_value[1] * 100 / 255)
                 )
         elif mode == MODE_PRESET:
             pat = self.effect
@@ -480,106 +612,133 @@ class LEDENETDevice:
         mode_str += utils.raw_state_to_dec(rx)
         return f"{power_str} [{mode_str}]"
 
-    def _set_power_state(self, new_power_state):
+    def _set_power_state(self, new_power_state: int) -> None:
         """Set the power state in the raw state."""
         self._replace_raw_state({"power_state": new_power_state})
         self._set_transition_complete_time()
 
-    def _replace_raw_state(self, new_states):
+    def _replace_raw_state(self, new_states: Dict[str, int]) -> None:
+        assert self.raw_state is not None
         _LOGGER.debug("%s: _replace_raw_state: %s", self.ipaddr, new_states)
         self._set_raw_state(
             self.raw_state._replace(**new_states), set(new_states.keys())
         )
 
-    def isOn(self):
+    def isOn(self) -> bool:
         return self.is_on
 
-    def getWarmWhite255(self):
+    def getWarmWhite255(self) -> int:
         if self.color_mode not in {COLOR_MODE_CCT, COLOR_MODE_DIM}:
             return 255
         return self.brightness
 
-    def getWhiteTemperature(self):
+    def getWhiteTemperature(self) -> Tuple[int, int]:
+        """Returns the color temp and brightness"""
         # Assume input temperature of between 2700 and 6500 Kelvin, and scale
         # the warm and cold LEDs linearly to provide that
+        assert self.raw_state is not None
+        assert isinstance(self.raw_state, LEDENETRawState)
         raw_state = self.raw_state
         temp, brightness = white_levels_to_color_temp(
             raw_state.warm_white, raw_state.cool_white
         )
         return temp, brightness
 
-    def getRgbw(self):
+    def getRgbw(self) -> Tuple[int, int, int, int]:
         """Returns red,green,blue,white (usually warm)."""
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255, 255)
         return self.rgbw
 
     @property
-    def rgbw(self):
+    def rgbw(self) -> Tuple[int, int, int, int]:
         """Returns red,green,blue,white (usually warm)."""
+        assert self.raw_state is not None
+        raw_state = self.raw_state
         return (
-            self.raw_state.red,
-            self.raw_state.green,
-            self.raw_state.blue,
-            self.raw_state.warm_white,
+            raw_state.red,
+            raw_state.green,
+            raw_state.blue,
+            raw_state.warm_white,
         )
 
-    def getRgbww(self):
+    def getRgbww(self) -> Tuple[int, int, int, int, int]:
         """Returns red,green,blue,warm,cool."""
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255, 255, 255)
         return self.rgbww
 
     @property
-    def rgbww(self):
+    def rgbww(self) -> Tuple[int, int, int, int, int]:
         """Returns red,green,blue,warm,cool."""
+        assert isinstance(self.raw_state, LEDENETRawState)
+        raw_state = self.raw_state
         return (
-            self.raw_state.red,
-            self.raw_state.green,
-            self.raw_state.blue,
-            self.raw_state.warm_white,
-            self.raw_state.cool_white,
+            raw_state.red,
+            raw_state.green,
+            raw_state.blue,
+            raw_state.warm_white,
+            raw_state.cool_white,
         )
 
-    def getRgbcw(self):
+    def getRgbcw(self) -> Tuple[int, int, int, int, int]:
         """Returns red,green,blue,cool,warm."""
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255, 255, 255)
         return self.rgbcw
 
     @property
-    def rgbcw(self):
+    def rgbcw(self) -> Tuple[int, int, int, int, int]:
         """Returns red,green,blue,cool,warm."""
+        assert isinstance(self.raw_state, LEDENETRawState)
+        raw_state = self.raw_state
         return (
-            self.raw_state.red,
-            self.raw_state.green,
-            self.raw_state.blue,
-            self.raw_state.cool_white,
-            self.raw_state.warm_white,
+            raw_state.red,
+            raw_state.green,
+            raw_state.blue,
+            raw_state.cool_white,
+            raw_state.warm_white,
         )
 
-    def getCCT(self):
+    def getCCT(self) -> Tuple[int, int]:
         if self.color_mode != COLOR_MODE_CCT:
             return (255, 255)
-        return (self.raw_state.warm_white, self.raw_state.cool_white)
+        assert isinstance(self.raw_state, LEDENETRawState)
+        raw_state = self.raw_state
+        return (raw_state.warm_white, raw_state.cool_white)
 
     @property
-    def speed(self):
-        if self.addressable or self.original_addressable:
+    def speed(self) -> int:
+        assert self.raw_state is not None
+        if self.protocol in ADDRESSABLE_PROTOCOLS:
             return self.raw_state.speed
         return utils.delayToSpeed(self.raw_state.speed)
 
-    def getSpeed(self):
+    def getSpeed(self) -> int:
         return self.speed
 
+    def _generate_random_levels_change(self) -> Tuple[bytearray, Dict[str, int]]:
+        """Generate a random levels change."""
+        channels = {STATE_WARM_WHITE}
+        if COLOR_MODES_RGB.intersection(self.color_modes):
+            channels = {STATE_RED, STATE_GREEN, STATE_BLUE}
+        elif COLOR_MODE_CCT in self.color_modes:
+            channels = {STATE_WARM_WHITE, STATE_COOL_WHITE}
+        return self._generate_levels_change(
+            {
+                channel: random.randint(0, 255) if channel in channels else None
+                for channel in CHANNEL_STATES
+            }
+        )
+
     def _generate_levels_change(
         self,
-        channels,
-        persist=True,
-        brightness=None,
-    ):
+        channels: Dict[str, Optional[int]],
+        persist: bool = True,
+        brightness: Optional[int] = None,
+    ) -> Tuple[bytearray, Dict[str, int]]:
         """Generate the levels change request."""
-        channel_map = CHANNEL_REMAP.get(self.model_num)
+        channel_map = self.model_data.channel_map
         if channel_map:
             mapped_channels = {
                 channel: channels[channel_map.get(channel, channel)]
@@ -595,8 +754,7 @@ class LEDENETDevice:
         w2 = mapped_channels[STATE_COOL_WHITE]
 
         if (r or g or b) and (w or w2) and not self.rgbwcapable:
-            print("RGBW command sent to non-RGBW device")
-            raise ValueError("RGBW command sent to non-RGBW device")
+            raise ValueError("RGB&CW command sent to non-RGB&CW device")
 
         if brightness is not None and r is not None and g is not None and b is not None:
             (r, g, b) = self._calculateBrightness((r, g, b), brightness)
@@ -642,6 +800,7 @@ class LEDENETDevice:
             f"0x{write_mode.value:02X}",
         )
 
+        assert self._protocol is not None
         msg = self._protocol.construct_levels_change(
             persist, r_value, g_value, b_value, w_value, w2_value, write_mode
         )
@@ -653,7 +812,7 @@ class LEDENETDevice:
             updates.update({"warm_white": w_value, "cool_white": w2_value})
         return msg, updates
 
-    def _set_transition_complete_time(self):
+    def _set_transition_complete_time(self) -> None:
         """Set the time we expect the transition will be completed.
 
         Devices fade to a specific state so we want to avoid
@@ -662,8 +821,9 @@ class LEDENETDevice:
         and the brightness values will be wrong until
         the transition completes.
         """
+        assert self.raw_state is not None
         latency = STATE_CHANGE_LATENCY
-        if self.addressable or self.original_addressable:
+        if self.protocol in ADDRESSABLE_PROTOCOLS:
             latency = ADDRESSABLE_STATE_CHANGE_LATENCY
         transition_time = latency + utils.speedToDelay(self.raw_state.speed) / 100
         self._transition_complete_time = time.monotonic() + transition_time
@@ -674,59 +834,72 @@ class LEDENETDevice:
             self._transition_complete_time,
         )
 
-    def getRgb(self):
+    def getRgb(self) -> Tuple[int, int, int]:
         if self.color_mode not in COLOR_MODES_RGB:
             return (255, 255, 255)
         return self.rgb
 
     @property
-    def rgb(self):
-        return (self.raw_state.red, self.raw_state.green, self.raw_state.blue)
+    def rgb(self) -> Tuple[int, int, int]:
+        assert self.raw_state is not None
+        raw_state = self.raw_state
+        return (raw_state.red, raw_state.green, raw_state.blue)
 
-    def _calculateBrightness(self, rgb, level):
+    @property
+    def rgb_unscaled(self) -> Tuple[int, int, int]:
+        """Return the unscaled RGB."""
+        r, g, b = self.rgb
+        hsv = colorsys.rgb_to_hsv(r / 255.0, g / 255.0, b / 255.0)
+        r_p, g_p, b_p = colorsys.hsv_to_rgb(hsv[0], hsv[1], 1)
+        return round(r_p * 255), round(g_p * 255), round(b_p * 255)
+
+    def _calculateBrightness(
+        self, rgb: Tuple[int, int, int], level: int
+    ) -> Tuple[int, int, int]:
         hsv = colorsys.rgb_to_hsv(*rgb)
-        return colorsys.hsv_to_rgb(hsv[0], hsv[1], level)
+        r, g, b = colorsys.hsv_to_rgb(hsv[0], hsv[1], level)
+        return int(r), int(g), int(b)
 
-    def setProtocol(self, protocol):
-        if protocol == PROTOCOL_LEDENET_ORIGINAL:
-            self._protocol = ProtocolLEDENETOriginal()
-        elif protocol == PROTOCOL_LEDENET_8BYTE:
-            self._protocol = ProtocolLEDENET8Byte()
-        elif protocol == PROTOCOL_LEDENET_9BYTE:
-            self._protocol = ProtocolLEDENET9Byte()
-        elif protocol == PROTOCOL_LEDENET_ADDRESSABLE:
-            self._protocol = ProtocolLEDENETAddressable()
-        elif protocol == PROTOCOL_LEDENET_ORIGINAL_ADDRESSABLE:
-            self._protocol = ProtocolLEDENETOriginalAddressable()
-        else:
+    def setProtocol(self, protocol: str) -> None:
+        cls = PROTOCOL_NAME_TO_CLS.get(protocol)
+        if cls is None:
             raise ValueError(f"Invalid protocol: {protocol}")
+        self._protocol = cls()  # type: ignore
 
-    def _set_protocol_from_msg(self, full_msg, fallback_protocol):
-        if self._is_addressable(full_msg[1]):
-            self._protocol = ProtocolLEDENETAddressable()
-        elif self._is_original_addressable(full_msg[1]):
-            self._protocol = ProtocolLEDENETOriginalAddressable()
-        # Devices that use an 9-byte protocol
-        elif self._uses_9byte_protocol(full_msg[1]):
-            self._protocol = ProtocolLEDENET9Byte()
-        else:
-            self._protocol = fallback_protocol
+    def _set_protocol_from_msg(
+        self,
+        full_msg: bytes,
+        fallback_protocol: str,
+    ) -> None:
+        self._model_num = full_msg[1]
+        self._model_data = get_model(self._model_num, fallback_protocol)
+        version_num = full_msg[10] if len(full_msg) > 10 else 1
+        self.setProtocol(self._model_data.protocol_for_version_num(version_num))
 
-    def _generate_preset_pattern(self, pattern, speed):
+    def _generate_preset_pattern(
+        self, pattern: int, speed: int, brightness: int
+    ) -> bytearray:
         """Generate the preset pattern protocol bytes."""
-        if self.original_addressable:
+        protocol = self.protocol
+        if protocol in OLD_EFFECTS_PROTOCOLS:
             if pattern not in ORIGINAL_ADDRESSABLE_EFFECT_ID_NAME:
                 raise ValueError("Pattern must be between 1 and 300")
-        elif self.addressable:
+        elif protocol in NEW_EFFECTS_PROTOCOLS:
             if pattern not in ADDRESSABLE_EFFECT_ID_NAME:
                 raise ValueError("Pattern must be between 1 and 100")
         else:
             PresetPattern.valtostr(pattern)
             if not PresetPattern.valid(pattern):
                 raise ValueError("Pattern must be between 0x25 and 0x38")
-        return self._protocol.construct_preset_pattern(pattern, speed)
+        if not (1 <= brightness <= 100):
+            raise ValueError("Brightness must be between 1 and 100")
+        self._last_effect_brightness = brightness
+        assert self._protocol is not None
+        return self._protocol.construct_preset_pattern(pattern, speed, brightness)
 
-    def _generate_custom_patterm(self, rgb_list, speed, transition_type):
+    def _generate_custom_patterm(
+        self, rgb_list: List[Tuple[int, int, int]], speed: int, transition_type: str
+    ) -> bytearray:
         """Generate the custom pattern protocol bytes."""
         # truncate if more than 16
         if len(rgb_list) > 16:
@@ -738,12 +911,14 @@ class LEDENETDevice:
         if len(rgb_list) == 0:
             raise ValueError("setCustomPattern requires at least one color tuples")
 
+        assert self._protocol is not None
         return self._protocol.construct_custom_effect(rgb_list, speed, transition_type)
 
-    def _effect_to_pattern(self, effect):
+    def _effect_to_pattern(self, effect: str) -> int:
         """Convert an effect to a pattern code."""
-        if self.addressable:
+        protocol = self.protocol
+        if protocol in NEW_EFFECTS_PROTOCOLS:
             return ADDRESSABLE_EFFECT_NAME_ID[effect]
-        if self.original_addressable:
+        if protocol in OLD_EFFECTS_PROTOCOLS:
             return ORIGINAL_ADDRESSABLE_EFFECT_NAME_ID[effect]
         return PresetPattern.str_to_val(effect)
