@@ -11,6 +11,15 @@ class LevelWriteMode(Enum):
 
 PRESET_MUSIC_MODE = 0x62
 
+ATTR_IPADDR = "ipaddr"
+ATTR_ID = "id"
+ATTR_MODEL = "model"
+ATTR_MODEL_NUM = "model_num"
+ATTR_VERSION_NUM = "version_num"
+ATTR_FIRMWARE_DATE = "firmware_date"
+ATTR_MODEL_INFO = "model_info"
+ATTR_MODEL_DESCRIPTION = "model_description"
+
 # Color modes
 COLOR_MODE_DIM = "DIM"
 COLOR_MODE_CCT = "CCT"
@@ -27,6 +36,8 @@ MAX_TEMP = 6500
 WRITE_ALL_COLORS = (LevelWriteMode.ALL, LevelWriteMode.COLORS)
 WRITE_ALL_WHITES = (LevelWriteMode.ALL, LevelWriteMode.WHITES)
 
+DEFAULT_RETRIES = 2
+
 # Modes
 MODE_SWITCH = "switch"
 MODE_COLOR = "color"
@@ -83,3 +94,6 @@ CHANNEL_STATES = {
     STATE_WARM_WHITE,
     STATE_COOL_WHITE,
 }
+
+
+EFFECT_RANDOM = "random"
