@@ -4,16 +4,18 @@ import select
 import socket
 import threading
 import time
+from typing import Dict, List, Optional, Tuple
 
-from .base_device import LEDENETDevice
+from .base_device import PROTOCOL_PROBES, LEDENETDevice
 from .const import (
+    DEFAULT_RETRIES,
+    EFFECT_RANDOM,
     STATE_BLUE,
     STATE_COOL_WHITE,
     STATE_GREEN,
     STATE_RED,
     STATE_WARM_WHITE,
 )
-from .protocol import ProtocolLEDENET8Byte, ProtocolLEDENETOriginal
 from .sock import _socket_retry
 from .timer import LedTimer
 from .utils import color_temp_to_white_levels, utils
@@ -24,32 +26,32 @@ _LOGGER = logging.getLogger(__name__)
 class WifiLedBulb(LEDENETDevice):
     """A LEDENET Wifi bulb device."""
 
-    def __init__(self, ipaddr, port=5577, timeout=5):
+    def __init__(self, ipaddr: str, port: int = 5577, timeout: int = 5) -> None:
         """Init and setup the bulb."""
         super().__init__(ipaddr, port, timeout)
-        self._socket = None
+        self._socket: Optional[socket.socket] = None
         self._lock = threading.Lock()
         self.setup()
 
-    def setup(self):
+    def setup(self) -> None:
         """Setup the connection and fetch initial state."""
-        self.connect(retry=2)
+        self.connect(retry=DEFAULT_RETRIES)
         self.update_state()
 
-    def _connect_if_disconnected(self):
+    def _connect_if_disconnected(self) -> None:
         """Connect only if not already connected."""
         if self._socket is None:
             self.connect()
 
-    @_socket_retry(attempts=0)
-    def connect(self):
+    @_socket_retry(attempts=0)  # type: ignore
+    def connect(self) -> None:
         self.close()
         self._socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         self._socket.settimeout(self.timeout)
         _LOGGER.debug("%s: connect", self.ipaddr)
         self._socket.connect((self.ipaddr, self.port))
 
-    def close(self):
+    def close(self) -> None:
         if self._socket is None:
             return
         try:
@@ -59,14 +61,15 @@ class WifiLedBulb(LEDENETDevice):
         finally:
             self._socket = None
 
-    def turnOn(self, retry=2):
+    def turnOn(self, retry: int = DEFAULT_RETRIES) -> None:
         self._change_state(retry=retry, turn_on=True)
 
-    def turnOff(self, retry=2):
+    def turnOff(self, retry: int = DEFAULT_RETRIES) -> None:
         self._change_state(retry=retry, turn_on=False)
 
-    @_socket_retry(attempts=2)
-    def _change_state(self, turn_on=True):
+    @_socket_retry(attempts=DEFAULT_RETRIES)  # type: ignore
+    def _change_state(self, turn_on: bool = True) -> None:
+        assert self._protocol is not None
         _LOGGER.debug("%s: Changing state to %s", self.ipaddr, turn_on)
         with self._lock:
             self._connect_if_disconnected()
@@ -90,60 +93,88 @@ class WifiLedBulb(LEDENETDevice):
             # it stalls
             self.close()
 
-    def setWarmWhite(self, level, persist=True, retry=2):
+    def setWarmWhite(
+        self, level: int, persist: bool = True, retry: int = DEFAULT_RETRIES
+    ) -> None:
         self.set_levels(w=utils.percentToByte(level), persist=persist, retry=retry)
 
-    def setWarmWhite255(self, level, persist=True, retry=2):
+    def setWarmWhite255(
+        self, level: int, persist: bool = True, retry: int = DEFAULT_RETRIES
+    ) -> None:
         self.set_levels(w=level, persist=persist, retry=retry)
 
-    def setColdWhite(self, level, persist=True, retry=2):
+    def setColdWhite(
+        self, level: int, persist: bool = True, retry: int = DEFAULT_RETRIES
+    ) -> None:
         self.set_levels(w2=utils.percentToByte(level), persist=persist, retry=retry)
 
-    def setColdWhite255(self, level, persist=True, retry=2):
+    def setColdWhite255(
+        self, level: int, persist: bool = True, retry: int = DEFAULT_RETRIES
+    ) -> None:
         self.set_levels(w2=level, persist=persist, retry=retry)
 
-    def setWhiteTemperature(self, temperature, brightness, persist=True, retry=2):
-        cold, warm = color_temp_to_white_levels(temperature, brightness)
+    def setWhiteTemperature(
+        self,
+        temperature: int,
+        brightness: int,
+        persist: bool = True,
+        retry: int = DEFAULT_RETRIES,
+    ) -> None:
+        warm, cold = color_temp_to_white_levels(temperature, brightness)
         self.set_levels(w=warm, w2=cold, persist=persist, retry=retry)
 
-    def setRgb(self, r, g, b, persist=True, brightness=None, retry=2):
+    def setRgb(
+        self,
+        r: int,
+        g: int,
+        b: int,
+        persist: bool = True,
+        brightness: Optional[int] = None,
+        retry: int = DEFAULT_RETRIES,
+    ) -> None:
         self.set_levels(r, g, b, persist=persist, brightness=brightness, retry=retry)
 
     def setRgbw(
         self,
-        r=None,
-        g=None,
-        b=None,
-        w=None,
-        persist=True,
-        brightness=None,
-        w2=None,
-        retry=2,
-    ):
-        return self.set_levels(r, g, b, w, w2, persist, brightness, retry=retry)
+        r: Optional[int] = None,
+        g: Optional[int] = None,
+        b: Optional[int] = None,
+        w: Optional[int] = None,
+        persist: bool = True,
+        brightness: Optional[int] = None,
+        w2: Optional[int] = None,
+        retry: int = 2,
+    ) -> None:
+        self.set_levels(r, g, b, w, w2, persist, brightness, retry=retry)
 
-    @_socket_retry(attempts=2)
     def set_levels(
         self,
-        r=None,
-        g=None,
-        b=None,
-        w=None,
-        w2=None,
-        persist=True,
-        brightness=None,
-    ):
-        msg, updates = self._generate_levels_change(
-            {
-                STATE_RED: r,
-                STATE_GREEN: g,
-                STATE_BLUE: b,
-                STATE_WARM_WHITE: w,
-                STATE_COOL_WHITE: w2,
-            },
-            persist,
-            brightness,
+        r: Optional[int] = None,
+        g: Optional[int] = None,
+        b: Optional[int] = None,
+        w: Optional[int] = None,
+        w2: Optional[int] = None,
+        persist: bool = True,
+        brightness: Optional[int] = None,
+        retry: Optional[int] = None,
+    ) -> None:
+        self._process_levels_change(
+            *self._generate_levels_change(
+                {
+                    STATE_RED: r,
+                    STATE_GREEN: g,
+                    STATE_BLUE: b,
+                    STATE_WARM_WHITE: w,
+                    STATE_COOL_WHITE: w2,
+                },
+                persist,
+                brightness,
+            ),
+            retry=retry,
         )
+
+    @_socket_retry(attempts=2)  # type: ignore
+    def _process_levels_change(self, msg: bytearray, updates: Dict[str, int]) -> None:
         # send the message
         with self._lock:
             self._connect_if_disconnected()
@@ -152,7 +183,8 @@ class WifiLedBulb(LEDENETDevice):
             if updates:
                 self._replace_raw_state(updates)
 
-    def _send_msg(self, bytes):
+    def _send_msg(self, bytes: bytearray) -> None:
+        assert self._socket is not None
         _LOGGER.debug(
             "%s => %s (%d)",
             self.ipaddr,
@@ -161,7 +193,8 @@ class WifiLedBulb(LEDENETDevice):
         )
         self._socket.send(bytes)
 
-    def _read_msg(self, expected):
+    def _read_msg(self, expected: int) -> bytearray:
+        assert self._socket is not None
         remaining = expected
         rx = bytearray()
         begin = time.monotonic()
@@ -170,7 +203,7 @@ class WifiLedBulb(LEDENETDevice):
             if timeout_left <= 0:
                 break
             try:
-                self._socket.setblocking(0)
+                self._socket.setblocking(False)
                 read_ready, _, _ = select.select([self._socket], [], [], timeout_left)
                 if not read_ready:
                     _LOGGER.debug(
@@ -192,17 +225,18 @@ class WifiLedBulb(LEDENETDevice):
                 _LOGGER.debug("%s: socket error: %s", self.ipaddr, ex)
                 pass
             finally:
-                self._socket.setblocking(1)
+                self._socket.setblocking(True)
         return rx
 
-    def getClock(self):
+    def getClock(self) -> Optional[datetime.datetime]:
         msg = bytearray([0x11, 0x1A, 0x1B, 0x0F])
         with self._lock:
             self._connect_if_disconnected()
+            assert self._protocol is not None
             self._send_msg(self._protocol.construct_message(msg))
             rx = self._read_msg(12)
         if len(rx) != 12:
-            return
+            return None
         year = rx[3] + 2000
         month = rx[4]
         date = rx[5]
@@ -211,12 +245,15 @@ class WifiLedBulb(LEDENETDevice):
         second = rx[8]
         # dayofweek = rx[9]
         try:
-            dt = datetime.datetime(year, month, date, hour, minute, second)
+            dt: Optional[datetime.datetime] = datetime.datetime(
+                year, month, date, hour, minute, second
+            )
         except Exception:
             dt = None
         return dt
 
-    def setClock(self):
+    def setClock(self) -> None:
+        assert self._protocol is not None
         msg = bytearray([0x10, 0x14])
         now = datetime.datetime.now()
         msg.append(now.year - 2000)
@@ -235,10 +272,10 @@ class WifiLedBulb(LEDENETDevice):
             # cycle the connection
             self.close()
 
-    def _determine_protocol(self):
+    def _determine_protocol(self) -> bytearray:
         """Determine the type of protocol based of first 2 bytes."""
         read_bytes = 2
-        for protocol_cls in (ProtocolLEDENET8Byte, ProtocolLEDENETOriginal):
+        for protocol_cls in PROTOCOL_PROBES:
             protocol = protocol_cls()
             with self._lock:
                 self._connect_if_disconnected()
@@ -254,21 +291,50 @@ class WifiLedBulb(LEDENETDevice):
                     protocol.state_response_length - read_bytes
                 )
                 if protocol.is_valid_state_response(full_msg):
-                    self._set_protocol_from_msg(full_msg, protocol)
+                    self._set_protocol_from_msg(full_msg, protocol.name)
                 return full_msg
         raise Exception("Cannot determine protocol")
 
-    def setPresetPattern(self, pattern, speed):
-        msg = self._generate_preset_pattern(pattern, speed)
+    def setPresetPattern(
+        self,
+        pattern: int,
+        speed: int,
+        brightness: int = 100,
+        retry: int = DEFAULT_RETRIES,
+    ) -> None:
+        self._set_transition_complete_time()
+        self._send_with_retry(
+            self._generate_preset_pattern(pattern, speed, brightness), retry=retry
+        )
+
+    def set_effect(
+        self,
+        effect: str,
+        speed: int,
+        brightness: int = 100,
+        retry: int = DEFAULT_RETRIES,
+    ) -> None:
+        """Set an effect."""
+        if effect == EFFECT_RANDOM:
+            self.set_random()
+            return
+        self.setPresetPattern(
+            self._effect_to_pattern(effect), speed, brightness, retry=retry
+        )
+
+    def set_random(self, retry: int = DEFAULT_RETRIES) -> None:
+        """Set levels randomly."""
+        self._process_levels_change(*self._generate_random_levels_change(), retry=retry)
+
+    @_socket_retry(attempts=2)  # type: ignore
+    def _send_with_retry(self, msg: bytearray) -> None:
+        """Send a message under the lock."""
         with self._lock:
             self._connect_if_disconnected()
             self._send_msg(msg)
 
-    def set_effect(self, effect, speed):
-        """Set an effect."""
-        return self.setPresetPattern(self._effect_to_pattern(effect), speed)
-
-    def getTimers(self):
+    def getTimers(self) -> List[LedTimer]:
+        assert self._protocol is not None
         msg = bytearray([0x22, 0x2A, 0x2B, 0x0F])
         resp_len = 88
         with self._lock:
@@ -291,7 +357,8 @@ class WifiLedBulb(LEDENETDevice):
 
         return timer_list
 
-    def sendTimers(self, timer_list):
+    def sendTimers(self, timer_list: List[LedTimer]) -> None:
+        assert self._protocol is not None
         # remove inactive or expired timers from list
         for t in timer_list:
             if not t.isActive() or t.isExpired():
@@ -322,32 +389,37 @@ class WifiLedBulb(LEDENETDevice):
             # not sure what the resp is, prob some sort of ack?
             self._read_msg(4)
 
-    @_socket_retry(attempts=2)
-    def query_state(self, led_type=None):
+    @_socket_retry(attempts=2)  # type: ignore
+    def query_state(self, led_type: Optional[str] = None) -> bytearray:
         if led_type:
             self.setProtocol(led_type)
         elif not self._protocol:
             return self._determine_protocol()
 
+        assert self._protocol is not None
         with self._lock:
             self.connect()
             self._send_msg(self._protocol.construct_state_query())
             return self._read_msg(self._protocol.state_response_length)
 
-    def update_state(self, retry=2):
+    def update_state(self, retry: int = 2) -> None:
         rx = self.query_state(retry=retry)
         if rx and self.process_state_response(rx):
             self.available = True
             return
         self.set_unavailable()
 
-    @_socket_retry(attempts=2)
-    def setCustomPattern(self, rgb_list, speed, transition_type):
+    def setCustomPattern(
+        self,
+        rgb_list: List[Tuple[int, int, int]],
+        speed: int,
+        transition_type: str,
+        retry: int = DEFAULT_RETRIES,
+    ) -> None:
         """Set a custom pattern on the device."""
-        msg = self._generate_custom_patterm(rgb_list, speed, transition_type)
-        with self._lock:
-            self._connect_if_disconnected()
-            self._send_msg(msg)
+        self._send_with_retry(
+            self._generate_custom_patterm(rgb_list, speed, transition_type), retry=retry
+        )
 
-    def refreshState(self):
+    def refreshState(self) -> None:
         return self.update_state()
